/*
    dbf2html.cpp - classe principale per l'applicazione DBF2HTML
    Copyright (c) 1997,1999 Simone Govoni
*/

#include <dir.h>
#include <dos.h>
#include <fstream.h>
#include <iostream.h>
#include <stdlib.h>
#include <string.h>
#pragma hdrstop

#include "types.h"
#include "utility.h"
#include "cmdline.h"
#include "applic.h"
#include "errhnd.h"
#include "dbf.h"
#include "html.h"
#include "fieldlst.h"
#include "dbf2html.h"
#include "dbstruct.h"

Dbf2HtmlApp::Dbf2HtmlApp(int argc,char *argv[])
    : Application(argc,argv,"b:c:Bdf:hlst:"), ErrorHandler(argv[0]), flist_delim(","),
      fl_allocated(FALSE), dbf_allocated(FALSE), html_allocated(FALSE),
      border_width(0), input_ext(".dbf"), output_ext(".htm"),
      sw_deleted(FALSE), sw_fieldlist(FALSE), sw_longdate(FALSE),
      sw_caption(FALSE), sw_structure(FALSE), sw_captbottom(FALSE),
      sw_title(FALSE)
{
    char base_name[MAXPATH];
    char c;
    banner();
    while ((c=cline->getopt())!=EOF)
        switch (c) {
            case 'b':
                // Imposta la larghezza del bordo della tabella
                border_width=atoi(cline->getoptarg());
                break;
            case 'B':
                // Fa visualizzare la didascalia sotto la tabella
                sw_captbottom=TRUE;
                break;
            case 'c':
                // Imposta il testo della didascalia
                sw_caption=TRUE;
                captionarg=cline->getoptarg();
                break;
            case 'd':
                // Ignora i record marcati come "deleted"
                sw_deleted=TRUE;
                break;
            case 'h':
                // Visualizza l'help sugli switch.
                // Il controllo non ritorna al programma.
                usage();
                break;
            case 'f':
                // Fieldlist specificata sulla linea di comando
                sw_fieldlist=TRUE;
                listarg=cline->getoptarg();
                break;
            case 'l':
                // Usare quattro cifre per l'anno
                sw_longdate=TRUE;
                break;
            case 's':
                // Richiesta la visualizzazione della struttura del DBF
                sw_structure=TRUE;
                break;
            case 't':
                // Imposta il titolo del documento HTML
                sw_title=TRUE;
                titlearg=cline->getoptarg();
                break;
            case '?':
                // Switch non riconosciuto
                fatal("unknown switch.",NULL);
                break;
            default:
                break;
        }
    country(0,&cinfo);
    if (cline->getoptind()<cline->getargc()) {
        addextension(infile_name,(cline->getargv())[cline->getoptind()],input_ext);
        // prepara il nome del file di output (HTML)
        if (!sw_structure) {
            if (cline->getoptind()==cline->getargc()-1)
                removeextension(base_name,infile_name);
            else
                strcpy(base_name,(cline->getargv())[cline->getoptind()+1]);
            addextension(outfile_name,base_name,output_ext);
        }
    }
    else
        fatal("missing file name (use /h to see the help).",NULL);
    is=new Dbf(infile_name);
    dbf_allocated=TRUE;
    if (is->geterror()!=Dbf::OK && is->geterror()!=Dbf::END_OF_FILE)
        fatal("an error occurred opening the input file",infile_name,NULL);
    if (!sw_structure) {
        os=new Html(outfile_name);
        html_allocated=TRUE;
        if (os->geterror()!=Html::OK)
            fatal("an error occurred opening the output file",outfile_name,NULL);
    }
}


Dbf2HtmlApp::~Dbf2HtmlApp()
{
    if (fl_allocated)
        delete flp;
    if (html_allocated)
        delete os;
    if (dbf_allocated)
        delete is;
}


// Restituisce il codice da inviare al sistema operativo
int Dbf2HtmlApp::main(void)
{
    if (!sw_structure) {
        Bool filter_ok=filter(os,is);
        if (!filter_ok)
            fatal("an error occurred processing the input file",infile_name,NULL);
    }
    else
        printstruct(&cout,is);
    return EXIT_SUCCESS;
}


// Visualizza il banner di apertura
void Dbf2HtmlApp::banner(void)
{
#ifndef DEBUG
    cerr << "DBF2HTML  Version 1.01 Copyright (c) 1997,1999 Simone Govoni\n";
#else
    cerr << "DBF2HTML *DEBUGGING* " << __DATE__ << " Copyright (c) 1997,1999 Simone Govoni\n";
#endif
}


// Visualizza l'help sugli switch
void Dbf2HtmlApp::usage(void)
{
    cout << "Syntax: DBF2HTML [switches] dbffile[.dbf] [htmlfile[.htm]]\n"
            "Switches:\n"
            "  /b width\tSet the table border width to <width>\n"
            "  /c caption\tSet the table caption to <caption>\n"
            "  /B\t\tPut the caption (/c) below the table (default: above)\n"
            "  /d\t\tSkip deleted records\n"
            "  /f fieldlist\tProcess only fields specified in the <fieldlist>\n"
            "\t\t(for example, /f2,5,7 process only the 2th, 5th and 7th field)\n"
            "  /h\t\tDisplay this help screen\n"
            "  /l\t\tUse the four digits format to display years\n"
            "  /s\t\tDisplay the DBF structure\n"
            "  /t title\tSet the HTML document title to <title>\n\n"
            "The following macros can be used inside the title or the caption:\n"
            "  $f  The file name of the DBF       $q  The '\"' character\n"
            "  $p  The '\%' character              $$  The '$' character\n";
    exit(EXIT_SUCCESS);
}


// Costruisce una lista di campi di default con tutti i campi del DBF
void Dbf2HtmlApp::makefieldlist(void)
{
    int fnum=is->fcount();
    flp=new Fieldlist(fnum);
    for (int i=1;i<=fnum;i++)
        flp->addfield(i);
}


// Fa il parsing di una lista di numeri di campo separati da virgole
void Dbf2HtmlApp::parsefieldlist(char *cdl)
{
    char *s;        // una copia della stringa di cui fare il parsing
    char *p;        // puntatore ai token della stringa da parserizzare
    int fieldcount; // numero di campi nella lista
    int listsize;   // dimensione in caratteri della lista sulla linea di comando
    int fieldno;    // indice del campo corrente
    listsize=strlen(cdl);
    fieldcount=1;   // "1" perche' fieldcount=(number of FLIST_DELIM)+1
    // conta il numero di campi specificati nella lista su linea di comando
    for (int i=0;i<listsize;i++)
        if (cdl[i]==flist_delim[0])
            fieldcount++;
    flp=new Fieldlist(fieldcount);
    s=new char[listsize+1];
    strcpy(s,cdl);
    // scansione della lista dei campi come viene passata dalla linea di comando
    p=strtok(s,flist_delim);
    while (p) {
        if ((fieldno=atoi(p))!=0)
            flp->addfield(fieldno);
        p=strtok(NULL,flist_delim);
    }
    delete [] s;
}


// Sceglie una modalita' di allineamento in base al tipo di un campo DBF
Html::Alignmode Dbf2HtmlApp::getalignmode(Dbf::Typecode ftype)
{
    Html::Alignmode amode;
    switch (ftype) {
        case Dbf::CHARACTER:
        case Dbf::DATE:
            amode=Html::LEFT;
            break;
        case Dbf::LOGICAL:
            amode=Html::CENTER;
            break;
        case Dbf::NUMERIC:
            amode=Html::RIGHT;
            break;
        default:
            amode=Html::LEFT;
            break;
    }
    return amode;
}


/*
    Espande le macro all'interno della stringa src; la stringa
    espansa viene memorizzata nel buffer dest, preventivamente allocato.
*/
void Dbf2HtmlApp::expandmacros(char *dest,const char *src,Bool title)
{
    int len=strlen(src);
    int j=0;    // dove scrivere il prossimo carattere dentro dest
    char c;     // carattere da copiare dentro dest
    char dbfname[MAXFILE+1];
    char dbfext[MAXEXT+1];
    Bool nameinserted;      // TRUE se e' stato appena espanso il nome del DBF
    for (int i=0;i<len;i++) {
        nameinserted=FALSE;
        if (src[i]=='$') {
            if (i<len-1) {
                switch (src[++i]) {
                    case 'f':   // nome del file DBF (senza il percorso)
                        fnsplit(infile_name,NULL,NULL,dbfname,dbfext);
                        if (strlen(dbfname)+strlen(dbfext)+j<=Dbf2HtmlApp::MAXEXPSTRING-1) {
                            strupr(dbfname);
                            strupr(dbfext);
                            int k;  // usata per scandire il nome del DBF
                            int dnamelen=strlen(dbfname);
                            int dextlen=strlen(dbfext);
                            // copia il nome base dentro dest
                            for (k=0;k<dnamelen;k++)
                                dest[j++]=dbfname[k];
                            // copia l'estensione dentro dest
                            for (k=0;k<dextlen;k++)
                                dest[j++]=dbfext[k];
                            nameinserted=TRUE;
                        }
                        else
                            if (title)
                                fatal("title too long.",NULL);
                            else
                                fatal("caption too long.",NULL);
                        break;
                    case 'q':   // apici
                        c='\"';
                        break;
                    case '$':   // simbolo "$"
                        c='$';
                        break;
                    case 'p':   // simbolo "%"
                        c='%';
                        break;
                    default:    // copia il carattere
                        c=src[i];
                        break;
                }
            }
            else
                c='$';
        }
        else
            c=src[i];
        /*
            Controlla che l'ultima operazione eseguita non sia stata
            l'espansione della macro $f (nome del file). Se $f e' stata
            usata allora non si deve inserire in dest il carattere c poiche'
            la copiatura del nome in dest e' gia' stata fatta.
        */
        if (!nameinserted)
            if (j<=Dbf2HtmlApp::MAXEXPSTRING-2)
                dest[j++]=c;
            else
                if (title)
                    fatal("title too long.",NULL);
                else
                    fatal("caption too long.",NULL);
    }
    dest[j]='\0';
}


/*
    Crea la tabella HTML a partire dal file DBF.
    is  e' lo stream di input associato al file DBF;
    os  e' lo stream di output associato al file HTML da creare;
*/
Bool Dbf2HtmlApp::filter(Html *os,Dbf *is)
{
    Bool ok=TRUE;
    if (!sw_fieldlist)
        // costruisci la lista dei campi di default
        makefieldlist();
    else
        parsefieldlist(listarg);
    fl_allocated=TRUE;      // a questo punto una lista di campi e' stata allocata
    is->setdeleted(sw_deleted);
    os->comment("HTML page produced by DBF2HTML (c) 1997 Simone Govoni");
    os->comment("      Please, retain the above copyright notice      ");
    if (sw_title) {
        // alloca il buffer per l'interpretazione delle macro nel titolo
        char *exptitle=new char[Dbf2HtmlApp::MAXEXPSTRING];
        expandmacros(exptitle,titlearg,TRUE);
        os->openhead();
        os->title(exptitle);
        os->closehead();
        delete [] exptitle;
    }
    os->openbody();
    os->opentable(border_width);
    if (sw_caption) {
        Html::Alignmode alcapt;     // allineamento della didascalia
        if (sw_captbottom)
            alcapt=Html::BOTTOM;
        else
            alcapt=Html::TOP;
        /*
            Alloca il buffer per l'interpretazione dei caratteri di escape
            nella didascalia.
        */
        char *expcaption=new char[Dbf2HtmlApp::MAXEXPSTRING];
        expandmacros(expcaption,captionarg,FALSE);
        os->caption(expcaption,alcapt);
        delete [] expcaption;
    }
    os->openrecord();
    // stampa intestazione
    Dbf::Typecode ftype;
    char *fcontent=new char[256];
    int i;          // numero dell'elemento corrente nella Fieldlist
    Bool endscan=FALSE;
    flp->start();
    while (!endscan)
        if ((i=flp->getnext())!=0) {
            ftype=is->fieldtype(i);
            if (is->printable(ftype)) {
                is->fieldname(fcontent,i);
                os->fielddescript(fcontent);
            }
        }
        else
            endscan=TRUE;
    os->closerecord();
    Html::Alignmode amode;
    while (!is->eof() && ok) {
        // stampa contenuto del record
        os->openrecord();
        Bool endscan=FALSE;
        flp->start();
        while (!endscan)
            if ((i=flp->getnext())!=0) {
                ftype=is->fieldtype(i);
                if (is->printable(ftype)) {
                    is->fieldget(fcontent,i);
                    amode=getalignmode(ftype);
                    alltrim(fcontent);
                    if(ftype==Dbf::DATE)
                        formatdate(fcontent,(Dateformat)(cinfo.co_date),cinfo.co_dtsep,sw_longdate);
                    os->fieldcontent(fcontent,amode);
                }
            }
            else
                endscan=TRUE;
        os->closerecord();
        if (os->geterror()!=Html::OK)
            ok=FALSE;
        else {
            is->skip();
            if (is->geterror()!=Dbf::OK && is->geterror()!=Dbf::END_OF_FILE)
                ok=FALSE;
        }
    }
    delete [] fcontent;
    os->closetable();
    os->closebody();
    return ok;
}
