/*
    html.h - definizione della classe Html per la creazione di file HTML
    Copyright (c) 1997 Simone Govoni
*/

/*
    Questo header deve essere incluso dopo i seguenti file:
    fstream.h
*/

#ifndef HTML_H
#define HTML_H

class Html {
public:
    // Codici d'errore
    enum Errorcode {OK=0,NOT_OPEN};
    // Tipi di allineamento
    enum Alignmode {LEFT=0,CENTER,RIGHT,TOP,BOTTOM};
    typedef struct {
        char xascii;
        char *htmlcode;
    } Entity;
private:
    static Entity entities[];
    const int entitynum;
    ofstream *ofs;          // stream di output
    Errorcode error;        // stato di errore dell'oggetto
    char *searchentity(unsigned char c);
public:
    Html(const char *filename);
    ~Html();

    /*
        Invia una stringa sullo stream di output convertendo i caratteri
        che hanno un significato particolare in HTML oppure con non sono
        ASCII standard.
    */
    void puttext(const char *s);

    // Restituisce il codice d'errore relativo all'ultima operazione
    Errorcode geterror(void) const { return error; }

    void comment(const char *s);
    void openhead(void);
    void closehead(void);
    void title(const char *s);
    void openbody(void);
    void closebody(void);
    void heading(int level,const char *s);
    void openpar(void);
    void closepar(void);
    void linebreak(void);
    void opencenter(void);
    void closecenter(void);
    void hrule(void);
    void hrule(int size);
    void opentable(int border=0);
    void closetable(void);
    void openrecord(void);
    void closerecord(void);
    void fielddescript(const char *s);
    void fieldcontent(const char *s,Html::Alignmode amode=Html::LEFT);
    void caption(char *s,Html::Alignmode alcapt=Html::TOP);
};

#endif  // HTML_H