/*
    utility.cpp - funzioni di utilita' varia
    Copyright (c) 1997 Simone Govoni
*/

#include <dir.h>
#include <string.h>
#pragma hdrstop

#include "types.h"
#include "utility.h"


// Aggiungi a src l'estensione, se non e' gia' stata aggiunta
void addextension(char *dst,const char *src,const char *ext)
{
    strcpy(dst,src);
    if (!(fnsplit(src,NULL,NULL,NULL,NULL) & EXTENSION))
        strcat(dst,ext);
}


// Rimuovi da src l'estensione, se presente
void removeextension(char *dst,const char *src)
{
    char drive[MAXDRIVE];
    char dir[MAXDIR];
    char name[MAXFILE];
    fnsplit(src,drive,dir,name,NULL);
    fnmerge(dst,drive,dir,name,NULL);
}


/*
    Rimuovi tutti gli spazi in testa e in coda di una stringa.
    NB: La stringa passata in input viene modificata.
*/
char *alltrim(char *s)
{
    int len;
    if ((len=strlen(s))>0) {
        // togli gli spazi in coda
        for (int i=len-1;s[i]==' ' && i>=0;i--)
            ;
        s[i+1]='\0';
        // togli gli spazi in testa
        for (i=0;s[i]==' ';i++)
            ;
        for (int j=i;s[j];j++)
            s[j-i]=s[j];
        s[j-i]='\0';
    }
    return s;
}


/*
    Aggiunge alla coda di s abbastanza spazi per rendere s lunga len
    caratteri. Il buffer s deve essere gia' opportunamente dimensionato
    per contenere la stringa allungata.
*/
void padblanks(char *s,int len)
{
    for (int i=strlen(s);i<len;i++)
        s[i]=' ';
    s[len]='\0';
}


/*
    Converte una data dal formato AAAAMMDD al formato nazionale corrente.
    Se longdate e' TRUE allora devono essere conservate tutte le quattro
    cifre dell'anno.
*/
void formatdate(char *date,Dateformat format,const char *sep,Bool longdate)
{
    char bufdate[11];
    if (strlen(date)==8) {
        strcpy(bufdate,date);
        if (format==JAPAN && longdate)
            date[4]=date[7]=sep[0];
        else
            date[2]=date[5]=sep[0];
        switch (format) {
            case USA:
                // USA (MM/DD/YY)
                date[0]=bufdate[4];
                date[1]=bufdate[5];
                date[3]=bufdate[6];
                date[4]=bufdate[7];
                if(!longdate) {
                    date[6]=bufdate[2];
                    date[7]=bufdate[3];
                }
                else {
                    date[6]=bufdate[0];
                    date[7]=bufdate[1];
                    date[8]=bufdate[2];
                    date[9]=bufdate[3];
                    date[10]='\0';
                }
                break;
            case EUROPE:
                // European (DD/MM/YY)
                date[0]=bufdate[6];
                date[1]=bufdate[7];
                date[3]=bufdate[4];
                date[4]=bufdate[5];
                if(!longdate) {
                    date[6]=bufdate[2];
                    date[7]=bufdate[3];
                }
                else {
                    date[6]=bufdate[0];
                    date[7]=bufdate[1];
                    date[8]=bufdate[2];
                    date[9]=bufdate[3];
                    date[10]='\0';
                }
                break;
            case JAPAN:
                // Japanese (YY/MM/DD)
                if(!longdate) {
                    date[0]=bufdate[2];
                    date[1]=bufdate[3];
                    date[3]=bufdate[4];
                    date[4]=bufdate[5];
                    date[6]=bufdate[6];
                    date[7]=bufdate[7];
                }
                else {
                    date[0]=bufdate[0];
                    date[1]=bufdate[1];
                    date[2]=bufdate[2];
                    date[3]=bufdate[3];
                    date[5]=bufdate[4];
                    date[6]=bufdate[5];
                    date[8]=bufdate[6];
                    date[9]=bufdate[7];
                    date[10]='\0';
                }
                break;
            default:
                break;
        }
    }
}
